// GPIOSample.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "Windev.h"

#include "gpio.h"

/*
	Available GPIO's in the EM-X270 Platform
	GPIO#	|AlternateFunction	|Available with Keypad working?		|Pin# on the keypad extender P2
	------------------------------------------------------------------------------------------
1	22		|KP_MKOUT<7>		|Yes - available when using keypad	|10
2	34		|KP_MKIN<3>			|Yes - available when using keypad	|1
3	36		|KP_MKIN<7>			|Yes - available when using keypad	|9
4	39		|KP_MKIN<4>			|Yes - available when using keypad	|3
5	91		|KP_MKIN<6>			|Yes - available when using keypad	|7
6	96		|KP_MKOUT<6>		|Yes - available when using keypad	|8
7	99		|KP_MKIN<5>			|Yes - available when using keypad	|5
8	100		|KP_MKIN<0>			|No - Keypad is using it for itself	|
9	101		|KP_MKIN<1>			|No - Keypad is using it for itself	|
10	102		|KP_MKIN<2>			|No - Keypad is using it for itself	|
11	103		|KP_MKOUT<0>		|No - Keypad is using it for itself	|
12	104		|KP_MKOUT<1>		|No - Keypad is using it for itself	|
13	105		|KP_MKOUT<2>		|No - Keypad is using it for itself	|
14	106		|KP_MKOUT<3>		|Yes - available when using keypad	|2
15	107		|KP_MKOUT<4>		|Yes - available when using keypad	|4
16	108		|KP_MKOUT<5>		|Yes - available when using keypad	|6

NOTES:
-	The column "Available with Keypad working":
		- If you choose the Keypad component from the Catalog then all GPIO's marked as not available - are not available
		- If you did not inlude Keypad in the Catalog then you can use the GPIO's marked as not available
*/



#define GPIO_22		22	//KP_MKOUT<7> - pin no. 10 in P2 (Demo Keypad)  //Will be set as output
#define GPIO_96		96	//KP_MKOUT<6> - pin no. 8 in P2 (Demo Keypad)	//Will be set as input
#define GPIO_36		36  //KP_MKIN<7> - pin no. 9 in P2 (Demo Keypad)	//Will be set as input
/************************************************************************************
This short program demonstrates how to use Compulab's GPIO Driver.
************************************************************************************/


int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	int iRet = TRUE;
	HANDLE hEvent; 
	GPIO_VALUE gpioVal;
	
	//Opens a handle to the gpio driver
	HANDLE hGpio = GPIOOpen();
	if (hGpio == NULL)
		return FALSE;

	//Check if the gpio we are about to use are available for us
	if (!GPIOIsAvailable(hGpio, GPIO_22) || !GPIOIsAvailable(hGpio,GPIO_36) || !GPIOIsAvailable(hGpio,GPIO_96))
	{
		printf("The gpios selected are not available\n");
		return 0;
	}
	//Letting the driver know we are going to use these GPIO's
	if (!GPIOAquireGpio(hGpio, GPIO_22) || !GPIOAquireGpio(hGpio, GPIO_36) || !GPIOAquireGpio(hGpio, GPIO_96))
	{
		printf("The gpios selected are not available\n");
		return 0;
	}
		
	//Sets gpio 36 as Input
	GPIOSetAsInput(hGpio, GPIO_36);
	
	//sets gpio 22 as Output with a logical value of 1
	GPIOSetAsOutput(hGpio, GPIO_22, GPIO_HIGH);
	

	//Sets up an interrupt event for GPIO 36 to react on falling and rising edge
		
	hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
	printf("Event hnd = 0x%x.\n",hEvent);

	//Initialize the GPIO_INTERRUPT struct
	GPIO_INTERRUPT intr;
	intr.dwGpioNum = GPIO_36;		
	intr.Edge = GPIO_REACT_ON_BOTH;		//Decides if we react on falling edge, rising edge or both.
	intr.Event = hEvent;				//The event the will be signaled when the edge will happen.
	iRet = GPIORegisterInterrupt(hGpio, intr);

	if (!iRet) {
		printf("Please contact Compulab if you ever reach this point with this demo\r\n");
		CloseHandle(hEvent);
		return -1;
	}

	GPIO_VALUE gpioVal22 = GPIO_HIGH;
	//This loop will continue until you trigger the edge event	
	while(TRUE)
	{
		//Waits for the edge event we just initialized. 
		iRet = WaitForSingleObject(hEvent, 3000);

		if (iRet == WAIT_TIMEOUT)
		{
			//No edge was detected	
			printf("GPIOSample: Interrupt timeout. Trigger an edge for gpio 36 to continue\n");
			//Invert the value of GPIO 22
			GPIOSetAsOutput(hGpio, GPIO_22, gpioVal22);
			gpioVal22 = (gpioVal22 == GPIO_HIGH) ? GPIO_LOW : GPIO_HIGH;
		}
		else
		{
			//Edge was detected. we can proceed 
			printf("GPIOSample: Interrupt event was signaled.\n");
			//This function will signal the kernel that interrupt processing has been completed.
			GPIOInterruptDone(hGpio, GPIO_36);
			break;
		}
	}

	//Unregisters the interrupt on gpio 36
	GPIOUnRegisterInterrupt(hGpio, GPIO_36);

	//This section demonstrates polling.
	//The loop will run until the logical value of gpio 96 is 0
	gpioVal = GPIO_HIGH;
	GPIOSetAsInput(hGpio, GPIO_96);
	printf("Polling GPIO 96, Assert a logical value 1 to continue.\n");
	while(gpioVal != GPIO_LOW)
	{
		if(GPIOGetValue(hGpio, GPIO_96, &gpioVal) == FALSE)
		{
			printf("Error: Unable to read gpio 96 value\n");
			break;
		}
		Sleep(100);		
	}

	//Letting the driver know we don't need the GPIO's anymore
	if (!GPIOReleaseGpio(hGpio, GPIO_22) || !GPIOReleaseGpio(hGpio, GPIO_36) || !GPIOReleaseGpio(hGpio, GPIO_96))
	{
		printf("The gpios selected are not available\n");
		return 0;
	}
	printf("That concludes the GPIO sample.\n");
	//Close the handle to the gpio driver
	GPIOClose(hGpio);

	if (hEvent)
		CloseHandle(hEvent);

	return 0;
}

